{% load static %}
<link rel="stylesheet" href="{% static "admin_tools/css/jquery/jquery-ui.css" %}" type="text/css" media="screen, projection"/>
<link rel="stylesheet" href="{% static "admin_tools/css/dashboard.css" %}" type="text/css" media="screen, projection"/>
<!--[if lt IE 8]>
<link rel="stylesheet" href="{% static "admin_tools/css/dashboard-ie.css" %}" type="text/css" media="screen, projection"/>
<![endif]-->
{% for media_type, files in css_files.items %}{% for css in files %}
<link rel="stylesheet" href="{% static css %}" type="text/css" media="{{ media_type  }}"/>{% endfor %}{% endfor %}
